@@ -112,9 +112,11 @@ extern "C" const char * console_prompt(const char * s);
 bool dfu_get_scriptstore_addr(size_t & start,size_t & taille);
 bool dfu_get_scriptstore(const char * fname);
 bool dfu_send_scriptstore(const char * fname);
+bool dfu_send_rescue(const char * fname);
 const int nwstoresize1=0x8000,nwstoresize2=0x8014;
 bool dfu_send_firmware(const char * fname);
 bool dfu_send_apps(const char * fname);
+bool dfu_update_khicas(const char * fname); 
 
 #if defined HAVE_LIBMICROPYTHON
 #include <string>
@@ -134,11 +136,59 @@ extern "C" size_t pythonjs_stack_size,pythonjs_heap_size;
 extern "C" void * bf_ctx_ptr;
 extern "C" size_t bf_global_prec;
 
+struct fileinfo_t {
+  std::string filename;
+  std::string type;
+  size_t size;
+  size_t header_offset;
+  int mode;
+};
+size_t tar_totalsize(const char * buffer,size_t byteLength);
+std::vector<fileinfo_t> tar_fileinfo(const char * buffer,size_t byteLength);
+// tar file format: operations on a malloc-ed char * buffer of size buffersize
+// (malloc is assumed if buffer needs to be resized by tar_adddata)
+extern char * buf64k; // a 64k buffer in RAM for flash sector copy
+extern int numworks_maxtarsize; // max tar size on the Numworks
+extern size_t tar_first_modified_offset; // will be used to truncate the file sent to  the Numworks
+int flash_adddata(const char * buffer_,const char * filename,const char * data,size_t datasize,int exec);
+int tar_adddata(char * & buffer,size_t * buffersizeptr,const char * filename,const char * data,size_t datasize,int exec=0); // filename is only used to fill the header
+int flash_addfile(const char * buffer,const char * filename);
+int tar_addfile(char * & buffer,const char * filename,size_t * buffersizeptr);
+const char * tar_loadfile(const char * buffer,const char * filename,size_t * len);
+int tar_filebrowser(const char * buf,const char ** filenames,int maxrecords,const char * extension);
+// by default removefile will mark the file as deleted (this requires 1 sector write)
+// if mark_only==2, this will undelete the file
+// int flash_removefile(const char * buffer,const char * filename,size_t * tar_first_modif_offsetptr,int mark_only=1);
+// write all changes made in records (filename and readable attribut)
+// returns 0 if there is a mismatch between buffer and finfo
+int flash_synchronize(const char * buffer,const std::vector<fileinfo_t> & finfo,size_t * tar_first_modif_offsetptr);
+ulonglong fromstring8(const char * ptr);
+std::string toString8(longlong chksum);
+
+// empty trash: files marked as non readable are really removed
+// this will do 1 sector write from first sector where a file is marked to be removed to the end 
+int flash_emptytrash(const char * buffer,size_t * tar_first_modif_offsetptr);
+int flash_emptytrash(const char * buffer,const std::vector<fileinfo_t> & finfo,size_t * tar_first_modif_offsetptr);
+
+int tar_removefile(char * buffer,const char * filename,size_t * tar_first_modif_offsetptr);
+int tar_savefile(char * buffer,const char * filename);
+std::vector<fileinfo_t> tar_fileinfo(const char * buffer,size_t byteLength);
+char * file_gettar(const char * filename);
+// same as file_gettar but returns an aligned pointer and sets freeptr to the address to be free-ed
+char * file_gettar_aligned(const char * filename,char * & freeptr);
+int file_savetar(const char * filename,char * buffer,size_t buffersize);
+#if !defined KHICAS && !defined USE_GMP_REPLACEMENTS && !defined GIAC_HAS_STO_38// 
+// numworks_gettar return 0 or a buffer of size numworks_maxtarsize
+// initialized with the calculator content
+char * numworks_gettar(size_t & tar_first_modif_offset); 
+bool numworks_sendtar(char * buffer,size_t buffersize,size_t tar_first_modif_offset=0);
+#endif
+
 #ifndef NO_NAMESPACE_GIAC
 namespace giac {
 #endif // ndef NO_NAMESPACE_GIAC
-  // 3 or 1 if a list of space separated commandnames includes buf
 
+  // 3 or 1 if a list of space separated commandnames includes buf
   int dichotomic_search(const char * const * tab,unsigned tab_size,const char * s);
   void opaque_double_copy(void * source,void * target);
   double opaque_double_val(const void * source);
@@ -530,11 +580,12 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     std::vector<unsigned char> data;
   };
   typedef std::map<std::string,nwsrec,ltstring> nws_map;
+  std::string dos2unix(const std::string & res);
   bool scriptstore2map(const char * fname,nws_map & m);
   bool map2scriptstore(const nws_map & m,const char * fname);
   // check that filename content matches a file content signed in sigfilename
   bool sha256_check(const char * sigfilename,const char * filename);
-  bool nws_certify_firmware(GIAC_CONTEXT); // Numworks certification
+  bool nws_certify_firmware(bool with_overwrite,GIAC_CONTEXT); // Numworks certification
 
   struct parser_lexer {
     int _index_status_; // 0 if [ -> T_VECT_DISPATCH, 1 if [ -> T_INDEX_BEGIN
@@ -552,6 +603,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     int _i_sqrt_minus1_;
   };
   std::string gen2string(const gen & g);
+  const int turtle_length=10;
 #ifdef KHICAS
   struct logo_turtle {
     double x,y;
@@ -559,13 +611,13 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     bool visible; // true if turtle visible
     bool mark; // true if moving marks
     bool direct; // true if rond/disque is done in the trigonometric direction
+    char turtle_width;
+    short int s;//std::string s;
     int color;
-    int turtle_length;
     int radius; // 0 nothing, >0 -> draw a plain disk 
     // bit 0-8=radius, bit9-17 angle1, bit 18-26 angle2, bit 27=1 filled  or 0 
     // <0 fill a polygon from previous turtle positions
-    int s;//std::string s;
-    logo_turtle(): x(100),y(100),theta(0),visible(true),mark(true),direct(true),color(0),turtle_length(10),radius(0) {}
+    logo_turtle(): x(100),y(100),theta(0),visible(true),mark(true),direct(true),color(0),turtle_width(1),radius(0) {}
   };
 #else // KHICAS
   struct logo_turtle {
@@ -575,16 +627,16 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     bool mark; // true if moving marks
     bool direct; // true if rond/disque is done in the trigonometric direction
     int color;
-    int turtle_length;
+    int turtle_width;
     int radius; // 0 nothing, >0 -> draw a plain disk 
     // bit 0-8=radius, bit9-17 angle1, bit 18-26 angle2, bit 27=1 filled  or 0 
     // <0 fill a polygon from previous turtle positions
     std::string s;
     void * widget;
 #ifdef IPAQ
-    logo_turtle(): x(70),y(70),theta(0),visible(true),mark(true),direct(true),color(0),turtle_length(10),radius(0),widget(0) {}
+    logo_turtle(): x(70),y(70),theta(0),visible(true),mark(true),direct(true),color(0),turtle_width(1),radius(0),widget(0) {}
 #else
-    logo_turtle(): x(100),y(100),theta(0),visible(true),mark(true),direct(true),color(0),turtle_length(10),radius(0),widget(0) {}
+    logo_turtle(): x(100),y(100),theta(0),visible(true),mark(true),direct(true),color(0),turtle_width(1),radius(0),widget(0) {}
 #endif
   };
 #endif // KHICAS
