@@ -19,6 +19,9 @@
 
 #ifndef __GRAPHE_H
 #define __GRAPHE_H
+#include <time.h>
+#include <stdarg.h>
+#include <math.h>
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -26,12 +29,15 @@
 #include "gen.h"
 #include "unary.h"
 #include "moyal.h"
+#include "optimization.h"
+#include "signalprocessing.h"
 #include <string>
 #include <iostream>
 #include <fstream>
 #include <queue>
 #include <stack>
 #include <set>
+#include <bitset>
 #ifdef HAVE_LIBGLPK
 #include <glpk.h>
 #endif
@@ -43,6 +49,7 @@
 #define PLASTIC_NUMBER_2 1.75487766625
 #define PLASTIC_NUMBER_3 2.32471795724
 #define MARGIN_FACTOR 0.139680581996 // pow(PLASTIC_NUMBER,-7)
+#define SIP_NBITS 64
 
 #ifndef NO_NAMESPACE_GIAC
 namespace giac {
@@ -50,6 +57,8 @@ namespace giac {
 
 typedef unsigned long ulong;
 
+#define _GT_HAAR_LIMIT 60
+
 enum gt_dot_token_type {
     _GT_DOT_TOKEN_TYPE_IDENTIFIER = 1,
     _GT_DOT_TOKEN_TYPE_NUMBER = 2,
@@ -64,6 +73,7 @@ enum gt_attribute {
     _GT_ATTRIB_COLOR,
     _GT_ATTRIB_SHAPE,
     _GT_ATTRIB_STYLE,
+    _GT_ATTRIB_WIDTH,
     _GT_ATTRIB_DIRECTED,
     _GT_ATTRIB_WEIGHTED,
     _GT_ATTRIB_POSITION,
@@ -90,6 +100,36 @@ enum gt_vertex_cover_algorithm {
     _GT_VC_EXACT
 };
 
+enum gt_conn_check {
+    _GT_CC_CONNECTED, // current subgraph is connected
+    _GT_CC_COMPONENTS_ARE_SUBGRAPHS, // components are subgraphs with indices 1,2,...
+    _GT_CC_FIND_COMPONENTS // split graph to connected components and iterate
+};
+
+/* special graph constructors */
+extern const unary_function_ptr * const at_graph;
+extern const unary_function_ptr * const at_cycle_graph;
+extern const unary_function_ptr * const at_path_graph;
+extern const unary_function_ptr * const at_complete_graph;
+extern const unary_function_ptr * const at_complete_binary_tree;
+extern const unary_function_ptr * const at_complete_kary_tree;
+extern const unary_function_ptr * const at_kneser_graph;
+extern const unary_function_ptr * const at_odd_graph;
+extern const unary_function_ptr * const at_johnson_graph;
+extern const unary_function_ptr * const at_star_graph;
+extern const unary_function_ptr * const at_wheel_graph;
+extern const unary_function_ptr * const at_web_graph;
+extern const unary_function_ptr * const at_prism_graph;
+extern const unary_function_ptr * const at_antiprism_graph;
+extern const unary_function_ptr * const at_grid_graph;
+extern const unary_function_ptr * const at_torus_grid_graph;
+extern const unary_function_ptr * const at_sierpinski_graph;
+extern const unary_function_ptr * const at_petersen_graph;
+extern const unary_function_ptr * const at_flower_snark;
+extern const unary_function_ptr * const at_goldberg_snark;
+extern const unary_function_ptr * const at_paley_graph;
+extern const unary_function_ptr * const at_haar_graph;
+
 class graphe {
 public:
     typedef std::vector<int> ivector;
@@ -101,7 +141,8 @@ public:
     typedef std::vector<dpair> dpairs;
     typedef std::vector<double> point;
     typedef std::vector<point> layout;
-    typedef std::map<int,std::map<int,ipair> > sparsemat;
+    typedef std::map<int,ipair> sparsematrow;
+    typedef std::map<int,sparsematrow> sparsemat;
     typedef std::set<ipair> edgeset;
     typedef std::vector<std::map<int,int> > edgemap;
     typedef std::vector<std::map<int,double> > edgemapd;
@@ -109,6 +150,9 @@ public:
     typedef std::vector<double> dvector;
     typedef std::set<int> iset;
     typedef std::vector<bool> bvector;
+    typedef std::vector<bvector> bvectors;
+    typedef std::vector<std::bitset<SIP_NBITS> > bitrow;
+    typedef std::vector<bitrow> bitmatrix;
 
     class vertex { // vertex class
         int m_subgraph;
@@ -250,7 +294,7 @@ public:
         ivector cover_number,initially_colored,branch_candidates,temp_colors,ordering;
         std::set<int> used_colors;
         int lb,ub,maxiter,nxcols;
-        bool generate_clique_cuts;
+        bool generate_clique_cuts,select_blb;
         glp_prob *mip;
         double timeout,*heur,*row_coeffs,*best_coeffs;
         int *row_indices,*best_indices;
@@ -264,7 +308,7 @@ public:
         static void callback(glp_tree *tree,void *info);
     public:
         painter(graphe *gr) { G=gr; }
-        int color_vertices(ivector &colors,const ivector &icol,int max_colors=0);
+        int color_vertices(ivector &colors,const ivector &icol,int max_colors=0,int tm_lim=0,bool verbose=false);
         int select_branching_variable(glp_tree *tree);
         void heur_solution(glp_tree *tree);
     };
@@ -274,7 +318,6 @@ public:
             /* arc struct holds only the edge information relevant for TSP */
             int head;
             int tail;
-            int sg_index;
         };
         enum solution_status {
             _GT_TSP_OPTIMAL,
@@ -287,62 +330,40 @@ public:
             _GT_TSP_FARTHEST_INSERTION_HEUR     = 2,
             _GT_TSP_FARTHEST_INSERTION_RANDOM   = 3
         };
-        graphe *G;                              // the graph
-        glp_prob *mip;                          // integer programming problem
-        bool isdirected;                        // true iff G is directed
-        bool isweighted;                        // true iff G is weighted
-        int sg;                                 // current subgraph index
-        std::set<ivector> subtours;             // subtours collected in during solving the last MIP
-        ivectors clustering_forest;             // hierarchical clustering forest of subgraphs
-        ivector tour,old_sol;                   // a tour, old mip solution
-        double *coeff;                          // coefficients to be passed to MIP solver
-        int *indices;                           // indices of row entries to be passed to MIP solver
-        bool *visited;                          // used to mark vertices as visited
-        arc *arcs;                              // arcs of G
-        int *sg_vertices;                       // list of sg_nv vertices of subgraph with index sg
-        int *sg_edges;                          // indices of edges belonging to the subgraph with index sg
-        int sg_nv;                              // number of vertices in subgraph with index sg
-        int sg_ne;                              // number of edges in subgraph with index sg
-        int nv;                                 // total number of vertices
-        int ne;                                 // total number of edges
-        int heur_type;                          // the type of heuristic to be applied
-        bool is_undir_weighted;                 // true iff G is undirected and weighted
-        bool is_symmetric_tsp;                  // true if G is undirected weighted clique
-        int num_nodes;                          // counting the hierarhical clustering forest nodes
-        solution_status status;                 // status of the solution
-        std::map<int,std::map<int,double> > weight_map;
-        std::map<int,std::map<int,double> > rlx_sol_map;
+        graphe *G;
+        glp_prob *mip;
+        bool isweighted,*visited,is_symmetric_tsp,verbose,cancellable;
+        std::set<ivector> subtours;
+        ivector tour,old_sol;
+        double *coeff,gap_tol;
+        arc *arcs;
+        int *indices,nv,ne,heur_type;
+        solution_status status;
+        std::map<int,std::map<int,double> > weight_map,rlx_sol_map;
         std::map<int,std::map<int,int> > loc_map;
-        dvector xev;
-        dvector obj;
+        dvector xev,obj;
         bvector can_branch;
         void formulate_mip();
         bool get_subtours();
         void add_subtours(const ivectors &sv);
-        void lift_subtours(ivectors &sv) const;
-        bool find_subgraph_subtours(ivectors &sv,solution_status &status);
+        bool find_tours(int k,ivectors &sv,solution_status &status);
         bool subtours_equal(const ivector &st1,const ivector &st2);
         ivector canonical_subtour(const ivector &subtour);
         void append_sce(const ivector &subtour);
-        void make_hierarchical_clustering_forest();
-        void hierarchical_clustering_dfs(int i,ivectors &considered_sec,ivectors &relevant_sec);
         ipair make_edge(int i,int j) const;
-        void make_sg_edges();
         int edge_index(const ipair &e);
-        int vertex_index(int i);
         double weight(int i,int j);
         double weight(const ipair &e) { return weight(e.first,e.second); }
         double lower_bound();
-        void perform_3opt_moves(ivector &hc);
-        void straighten(ivector &hc);
         bool is_move_feasible(int k,const ivector &t,const ipairs &x);
-        void lin_kernighan(ivector &hc);
-        bool make_3opt_moves(ivector &hc);
-        void improve_tour(ivector &hc);
+        bool lin_kernighan(ivector &hc);
+        bool perform_3opt_moves(ivector &hc);
+        void straighten(ivector &hc);
+        void improve_tour(ivector &hc,bool do_3opt=true);
         void farthest_insertion(int index,ivector &hc);
-        void christofides(ivector &hc);
+        bool christofides(ivector &hc,bool show_progress=false);
         static void sample_mean_stddev(const dvector &sample,double &mean,double &stddev);
-        void min_weight_matching_bipartite(const ivector &eind,const dvector &weights,ivector &matched_arcs);
+        bool min_weight_matching_bipartite(const ivector &eind,const dvector &weights,ivector &matched_arcs,bool msg=false);
         void select_branching_variable(glp_tree *tree);
         void rowgen(glp_tree *tree);
         void heur(glp_tree *tree);
@@ -360,10 +381,11 @@ public:
         int minimal_cut(int nn,int nedg,const ivector &beg,
                         const ivector &end,const ivector &cap,ivector &cut);
     public:
-        tsp(graphe *gr);
+        tsp(graphe *gr,double gap_tolerance=0,bool is_verbose=false);
         ~tsp();
+        int solve(int k,ivectors &hcv,dvector &costs);
         int solve(ivector &hc,double &cost);
-        double approx(ivector &hc);
+        bool approx(ivector &hc,double &ratio);
         double tour_cost(const ivector &hc);
     };
     
@@ -372,12 +394,14 @@ public:
         glp_prob *mip;
         ipairs mia; // must include arcs
         ivectors ft; // forbidden tours
-        bool isweighted;
+        bool isweighted,select_blb,verbose,terminated;
+        double gap_tol;
         void formulate_mip();
+        static void callback(glp_tree *tree,void *info);
     public:
-        atsp(graphe *gr,const ipairs &must_include_arcs);
+        atsp(graphe *gr,const ipairs &must_include_arcs,double gap_tolerance=0,bool is_verbose=false);
         ~atsp();
-        bool solve(ivector &hc,double &cost); // find shortest tour
+        bool solve(ivector &hc,double &cost); // find the shortest tour
         void ksolve(int k,ivectors &hcv,dvector &costs); // find k shortest tours
     };
 
@@ -393,19 +417,21 @@ public:
         void make_vertex_fixed(glp_prob *p,int j,bool in_cover);
         void find_mirrors(int v);
         void packing(glp_tree *tree);
-        ivector mirrors;
+        ivector mirrors,V,V_pos;
         double *heur_sol;
-        bool compute_heur;
-        bool is_k_vc;
-        ivector V,V_pos;
+        bool compute_heur,is_k_vc;
         ipairs edges;
-        int sg;
-        int last_row;
+        int sg,last_row;
     public:
         mvc_solver(graphe *gr,int s=-1);
         ~mvc_solver();
-        int solve(ivector &cover,int k=-1);
+        int solve(ivector &cover,int k=-1,int tm_lim=0,double gap_tol=0,bool verbose=false);
     };
+
+    typedef struct { double rhs, pi; } mcf_v_data;
+    typedef struct { double low, cap, cost, x; } mcf_a_data;
+    typedef struct { int set; } wbm_v_data;
+    typedef struct { double cost; int x; } wbm_a_data;
 #endif
 
     class rectangle { // simple rectangle class
@@ -517,11 +543,10 @@ public:
     class ostergard { // clique maximizer
         graphe *G;
         int maxsize;
-        bool found;
+        bool found,timed_out;
         double timeout; // seconds
         ivector c,incumbent,clique_nodes;
         clock_t start;
-        bool timed_out;
         void recurse(ivector &U,int size,ivector &position);
     public:
         ostergard(graphe *gr,double max_time=0) { G=gr; timeout=max_time; }
@@ -536,7 +561,7 @@ public:
             std::vector<tree_node*> children;
         } tree_node;
         struct kspaths_comparator {
-            bool operator()(const std::pair<gen,tree_node*> &a,const std::pair<gen,tree_node*> &b) {
+            bool operator()(const std::pair<gen,tree_node*> &a,const std::pair<gen,tree_node*> &b) const {
                 if (is_zero(a.first-b.first))
                     return a.second<b.second;
                 return is_strictly_greater(b.first,a.first,context0);
@@ -560,13 +585,8 @@ public:
     class mm { // An efficient implementation of Edmonds' blossom algorithm
         enum label_t { EVEN=0, ODD=1 };
         graphe *G;
-        int *mate;
-        int *label;
-        int *pred;
-        int *bridge;
+        int *mate,*label,*pred,*bridge,V,s;
         std::queue<int> Q;
-        int V;
-        int s;
         unionfind *ds;
         ivector ap;
         bool alternating_forest();
@@ -584,17 +604,44 @@ public:
         ~mm();
         void find_maximum_matching(ipairs &matching,int sg=-1);
     };
+
+    class sip { // subgraph isomorphism via Ullmann's algorithm
+        int N,n,max_sg,nb;
+        bitmatrix A,AT,M;
+        bvectors B;
+        bvector used_cols;
+        ivectors found;
+        bool induced,dir,isom;
+        ipairs changes;
+        std::stack<ipairs> snapshots;
+        static void clear_bitrow(bitrow &row);
+        static bool get_j(const bitrow &row,int j) { return row[j/SIP_NBITS].test(j%SIP_NBITS); }
+        static bool get_ij(const bitmatrix &mat,int i,int j) { return get_j(mat[i],j); }
+        static void set_ij(bitmatrix &mat,int i,int j,bool val) { mat[i][j/SIP_NBITS].set(j%SIP_NBITS,val); }
+        bool mult_bitrows(const bitrow &r,const bitrow &c);
+        bool M_has_empty_row();
+        bool recurse(int i=0);
+        void prune();
+        void edit(int i,int j) { set_ij(M,i,j,false); changes.push_back(make_pair(i,j)); }
+        void push_changes() { snapshots.push(changes); changes.clear(); }
+        void revert_changes();
+        bool is_isomorphism();
+    public:
+        sip(const graphe &G,const graphe &P,int max_sg_in=1);
+        int find_subgraphs(bool only_induced=true);
+        const ivector &get_subgraph(int i) const { return found[i]; }
+    };
     
     struct edges_comparator { // for sorting edges by their weight
         graphe *G;
-        bool operator()(const ipair &a,const ipair &b) {
+        bool operator()(const ipair &a,const ipair &b) const {
             return is_strictly_greater(G->weight(b),G->weight(a),G->giac_context());
         }
         edges_comparator(graphe *gr) { G=gr; }
     };
 
     struct ivectors_comparator { // for sorting ivectors by their length
-        bool operator()(const ivector &a,const ivector &b) {
+        bool operator()(const ivector &a,const ivector &b) const {
             return a.size()<b.size();
         }
     };
@@ -602,7 +649,7 @@ public:
     struct degree_comparator { // for sorting vertices by their degrees
         graphe *G;
         bool asc;
-        bool operator()(int v,int w) {
+        bool operator()(int v,int w) const {
             return (asc && G->degree(v)<G->degree(w)) ||
                     (!asc && G->degree(v)>G->degree(w));
         }
@@ -611,7 +658,7 @@ public:
 
     struct ivectors_degree_comparator { // for sorting sets of vertices by ascending total degree
         graphe *G;
-        bool operator()(const ivector &a,const ivector &b) {
+        bool operator()(const ivector &a,const ivector &b) const {
             int deg_a=0,deg_b=0;
             for (ivector_iter it=a.begin();it!=a.end();++it) {
                 deg_a+=G->degree(*it);
@@ -624,6 +671,7 @@ public:
         ivectors_degree_comparator(graphe *gr) { G=gr; }
     };
 
+    /* iterators */
     typedef std::vector<vertex>::const_iterator node_iter;
     typedef std::map<int,attrib>::const_iterator neighbor_iter;
     typedef attrib::const_iterator attrib_iter;
@@ -636,6 +684,7 @@ public:
     typedef dvector::const_iterator dvector_iter;
     typedef iset::const_iterator iset_iter;
 
+    /* static variables and constants */
     static const gen FAUX;
     static const gen VRAI;
     static bool verbose;
@@ -649,7 +698,8 @@ public:
     static std::map<ivector,std::vector<cpol> > cache;
     // special graphs
     static const int clebsch_graph[];
-    static const char* coxeter_graph[];
+    static const int coxeter_graph[];
+    static const char* coxeter_graph_vnames[];
     static const int dyck_graph[];
     static const int grinberg_graph[];
     static const int grotzsch_graph[];
@@ -689,7 +739,7 @@ public:
     static const int tutte_12cage_lcf[];
     static const int tutte_8cage_lcf[];
     static const int f26a_graph_lcf[];
-    static const int tutte_fragment_graph[];
+    static const int tutte_graph[];
     static const int brinkmann_graph[];
     static const int barnette_bosak_lederberg_graph[];
     static const int double_star_snark[];
@@ -699,7 +749,7 @@ public:
     static const int wong_graph[];
     static const char* const gewirtz_words[];
     static const int harborth_graph[];
-    static const int kittel_graph[];
+    static const int kittell_graph[];
     static const int krackhardt_kite_graph[];
     static const int meredith_graph[];
     static const int perkel_graph[];
@@ -708,6 +758,13 @@ public:
     static const int watkins_snark[];
     static const int wells_graph[];
     static const int wiener_araya_graph[];
+    static const int markstroem_graph[];
+
+    struct spcgraph { const char* name; int nv; int ne; };
+    static const spcgraph special_graph[];
+
+    static const std::ios::iostate cout_rdstate;
+    static const std::ios::iostate cerr_rdstate;
 
 private:
     const context *ctx;
@@ -730,6 +787,8 @@ private:
     void message(const char *format,int a) const;
     void message(const char *format,int a,int b) const;
     void message(const char *format,int a,int b,int c) const;
+    void suspend_logging();
+    void restore_logging();
     std::string giac_version() const;
     vertex &node(int i) { return nodes[i]; }
     bool dot_parse_attributes(std::ifstream &dotfile,attrib &attr);
@@ -738,7 +797,7 @@ private:
     static bool genmap2attrib(const gen_map &m,attrib &attr);
     static void attrib2genmap(const attrib &attr,gen_map &m);
     static void copy_attributes(const attrib &src,attrib &dest);
-    void write_attrib(std::ofstream &dotfile,const attrib &attr) const;
+    void write_attrib(std::ofstream &dotfile,const attrib &attr,bool style=true) const;
     static ivector_iter binsearch(ivector_iter first,ivector_iter last,int a);
     static size_t sets_union(const iset &A,const iset &B,iset &U);
     static size_t sets_intersection(const iset &A,const iset &B,iset &I);
@@ -767,8 +826,10 @@ private:
     static double layout_min(const layout &x,int d);
     static double layout_diameter(const layout &x);
     static void point2polar(point &p,double &r,double &phi);
+    vecteur draw_edge(int i,int j,const layout &x) const;
     static bool sparse_matrix_element(const sparsemat &A,int i,int j,ipair &val);
     static void multiply_sparse_matrices(const sparsemat &A,const sparsemat &B,sparsemat &P,int ncols,bool symmetric=false);
+    static gen sparse_product_element(const sparsemat &A, const sparsemat &B,int i,int j);
     static void transpose_sparsemat(const sparsemat &A,sparsemat &T);
     void multilevel_recursion(layout &x,int d,double R,double K,double tol,int depth=0);
     int mdeg(const ivector &V,int i) const;
@@ -788,13 +849,9 @@ private:
     static bool segments_crossing(const point &p,const point &r,const point &q,const point &s,point &crossing);
     static bool point2segment_projection(const point &p,const point &q,const point &r,point &proj);
     void force_directed_placement(layout &x,double K,double R=DBL_MAX,double tol=0.01,bool ac=true);
-    static bool get_node_position(const attrib &attr,point &p);
+    static bool get_node_position(const attrib &attr,point &p,GIAC_CONTEXT);
     void coarsening_mis(const ivector &V,graphe &G,sparsemat &P) const;
     void coarsening_ec(const ipairs &M,graphe &G,sparsemat &P) const;
-    int best_quadrant(const point &p,const layout &x) const;
-    void append_segment(vecteur &drawing, const point &p,const point &q,int color,int width,int style,bool arrow=false) const;
-    void append_node(vecteur &drawing,const point &p,int color,int width,int shape) const;
-    void append_label(vecteur &drawing,const point &p,const gen &label,int quadrant,int color=_BLACK) const;
     static int face_has_edge(const ivector &face,int i,int j);
     int first_neighbor_from_subgraph(const vertex &v,int sg) const;
     void set_nodes_embedded(const ivector &v,bool yes=true);
@@ -813,7 +870,7 @@ private:
     void tree_height_dfs(int i,int level,int &depth);
     void make_product_nodes(const graphe &G,graphe &P) const;
     static void extract_path_from_cycle(const ivector &cycle,int i,int j,ivector &path);
-    static void generate_nk_sets(int n,int k,std::vector<ulong> &v);
+    static void generate_nk_sets(int n,int k,std::vector<std::bitset<32> > &v);
     void strongconnect_dfs(ivectors &components,bvector &onstack,int i,int sg);
     bool degrees_equal(const ivector &v,int deg=0) const;
     void lca_recursion(int u,const ipairs &p,ivector &lca,unionfind &ds);
@@ -830,8 +887,8 @@ private:
     int uncolored_degree(const vertex &v) const;
     bool is_partially_colored() const;
     void remove_maximal_clique(iset &V) const;
-    bool bipartite_matching_bfs(ivector &dist);
-    bool bipartite_matching_dfs(int u,ivector &dist);
+    bool bipartite_matching_bfs(ivector &dist,int sg=-1);
+    bool bipartite_matching_dfs(int u,ivector &dist,int sg=-1);
     static gen make_colon_label(const ivector &v);
     void simplify(graphe &G,bool color_temp_vertices=false) const;
     intpoly tutte_poly_recurse(int vc);
@@ -856,28 +913,23 @@ private:
     bool hamcycle_recurse(ivector &path,int pos);
     void grasp_construct(double aplha, ivector &Q,bool cmpl,int sg);
     void grasp_local(ivector &Q,bool cmpl,int sg);
-    bool mvc_special(ivector &cover,const ivector &component,int sg);
+    bool mvc_special(ivector &cover,int sg);
     bool mvc_is_unconfined(int i,int sg=0) const;
     bool mvc_is_dominant(int v,int sg) const;
-    void mvc_reduce_basic(int &sg,bool assert_conn=false);
+    bool mvc_reduce_basic(int sg,int c);
     void mvc_half_integral(int sg, ivector &in_cover, ivector &out_cover);
     void mvc_alom(ivector &cover,int sg=-1);
     void mvc_dfs(ivector &cover,int sg=-1);
-    void mvc_bipartite(const ivector &U,const ivector &V,ivector &cover,int sg=-1);
-    ivector alom_candidates(const ivector &V,const vecteur &ds);
+    void mvc_bipartite(const ivector &U,const ivector &V,ivector &cover,int sg=-1,bool conn=false);
+    void alom_candidates(const ivector &V,ivector &cand,int sg);
     int count_edges_in_Nv(int v,int sg=-1) const;
     int count_edges(const ivector &V) const;
-    void make_hoffman_singleton_graph();
-    void make_higman_sims_graph();
-    void make_brouwer_haemers_graph();
-    void make_gewirtz_graph();
-    void make_schlaefli_graph();
-    void make_gosset_graph();
+    bool is_simplicial(int i,const sparsemat &A,double D=0.0);
 
 public:
     graphe(const context *contextptr=context0,bool support_attributes=true);
     graphe(const graphe &G);
-    graphe(const std::string &name,const context *contextptr=context0);
+    graphe(const std::string &name,const context *contextptr=context0,bool support_attributes=true);
     graphe &operator =(const graphe &other);
     bool is_simple() const;
     
@@ -886,7 +938,6 @@ public:
     double rand_uniform() const;
     double rand_normal() const { return giac::randNorm(ctx); }
     ivector rand_permu(int n) const;
-    static bool is_real_number(const gen &g);
     static gen to_binary(int number,int chars);
     const context *giac_context() const { return ctx; }
     static gen make_idnt(const char* name,int index=-1,bool intern=true);
@@ -898,12 +949,16 @@ public:
     static std::string genstring2str(const gen &g);
     static gen plusinf();
     void ivectors2vecteur(const ivectors &v,vecteur &res,bool sort_all=false) const;
+    void append_segment(vecteur &drawing, const point &p,const point &q,int color,int width,int style,bool arrow=false) const;
+    void append_node(vecteur &drawing,const point &p,int color,int width,int shape) const;
+    void append_label(vecteur &drawing,const point &p,const gen &label,int quadrant,int color=_BLACK) const;
     void reserve_nodes(int n) { assert(nodes.empty()); nodes.reserve(n); }
     bool read_gen(const gen &g);
     void read_special(const int *special_graph);
     void read_special(const char **special_graph);
     void copy(graphe &G) const;
     void copy_nodes(const std::vector<vertex> &V);
+    void copy_nodes(graphe &G,map<int,int> &vmap,int sg=-1) const;
     bool supports_attributes() const { return m_supports_attributes; }
     void clear();
     void clear_maximal_cliques() { maxcliques.clear(); }
@@ -929,12 +984,14 @@ public:
     bool is_edge_visited(const ipair &e) const { return is_edge_visited(e.first,e.second); }
     void unvisit_all_edges() { visited_edges.clear(); }
     gen to_gen();
-    int *to_array(int &sz,bool reduce=false) const;
+    int *to_array(int &sz,bool colored,bool reduce=false) const;
     bool write_latex(const std::string &filename,const gen &drawing) const;
-    bool write_dot(const std::string &filename) const;
+    bool write_dot(const std::string &filename,bool style=true) const;
+    bool write_lst(const std::string &filename) const;
     bool read_dot(const std::string &filename);
+    bool read_lst(const std::string &filename);
     bool is_null() const { return nodes.empty(); }
-    bool is_empty() const;
+    bool is_empty(int sg=-1) const;
     void weight_matrix(matrice &W) const;
     gen weight(int i,int j) const;
     gen weight(const ipair &edge) const { return weight(edge.first,edge.second); }
@@ -981,6 +1038,7 @@ public:
     const attrib &graph_attributes() const { return attributes; }
     const attrib &node_attributes(int i) const;
     const attrib &edge_attributes(int i,int j) const;
+    attrib &node_attributes(int i);
     attrib &edge_attributes(int i,int j);
     const attrib &edge_attributes(const ipair &e) const { return edge_attributes(e.first,e.second); }
     attrib &edge_attributes(const ipair &e) { return edge_attributes(e.first,e.second); }
@@ -996,8 +1054,8 @@ public:
     void remove_temporary_edges();
     bool remove_edge(int i,int j);
     bool remove_edge(const ipair &p) { return remove_edge(p.first,p.second); }
-    bool has_edge(int i,int j,int sg=-1) const;
-    bool has_edge(const ipair &p,int sg=-1) const { return has_edge(p.first,p.second,sg); }
+    bool has_edge(int i,int j) const;
+    bool has_edge(const ipair &p) const { return has_edge(p.first,p.second); }
     ipair make_edge(const vecteur &v) const;
     bool edges2ipairs(const vecteur &E,ipairs &ev,bool &notfound) const;
     vecteur ipairs2edges(const ipairs &E) const;
@@ -1006,13 +1064,12 @@ public:
     int in_degree(int index,int sg=-1) const;
     int out_degree(int index,int sg=-1) const;
     int degree(int index,int sg=-1) const;
-    int maximum_degree() const;
-    int minimum_degree() const;
-    int average_degree() const;
-    vecteur degree_sequence(int sg=-1) const;
-    void sort_by_degrees();
+    int maximum_degree(int sg=-1) const;
+    int minimum_degree(int sg=-1) const;
+    ivector degree_sequence(int sg=-1) const;
+    void sort_by_degrees(ivector &sigma);
     void adjacency_matrix(matrice &m) const;
-    void adjacency_sparse_matrix(sparsemat &sm,bool diag_ones=false) const;
+    void adjacency_sparse_matrix(sparsemat &sm,bool diag_ones=false,int sg=-1) const;
     void laplacian_matrix(matrice &m,bool normalize=false) const;
     void incidence_matrix(matrice &m) const;
     void set_graph_attribute(int key,const gen &val) { attributes[key]=val; }
@@ -1039,44 +1096,39 @@ public:
     int is_regular(int d) const;
     bool is_strongly_regular(ipair &sig);
     bool is_equal(const graphe &G) const;
-    void underlying(graphe &G) const;
-    void complement(graphe &G) const;
+    void underlying(graphe &G,int sg=-1) const;
+    void complement(graphe &G,int sg=-1) const;
     bool isomorphic_copy(graphe &G,const ivector &sigma,bool strip_attributes=false);
     bool relabel_nodes(const vecteur &labels);
     void induce_subgraph(const ivector &vi,graphe &G) const;
     void extract_subgraph(const ipairs &E,graphe &G) const;
     void subgraph_indices(const graphe &S,const ivector &v,ivector &sv) const;
     bool is_subgraph(const graphe &G) const;
+    void renumber_subgraph(int sg,int sg_new);
     void maximal_independent_set(ivector &ind) const;
-    void find_maximum_matching(ipairs &M);
-    void find_maximal_matching(ipairs &matching,int sg=-1) const;
-    bool trail(const vecteur &v);
+    void maximum_matching(ipairs &matching,int sg=-1,gt_conn_check cc=_GT_CC_FIND_COMPONENTS);
+    void maximal_matching(ipairs &matching,int sg=-1) const;
     bool demoucron(ivectors &faces,int sg=-1);
     void create_random_layout(layout &x,int dim);
     void make_spring_layout(layout &x,int d,double tol=0.001);
     void make_circular_layout(layout &x,const ivector &hull,double A=0,double tol=0.005,double elongate=0.0);
     void make_tutte_layout(layout &x,const ivector &outer_face);
-    bool make_planar_layout(layout &x);
+    bool make_planar_layout(layout &x,double *score=NULL);
+    bool make_best_planar_layout(layout &x,int iter);
     void make_tree_layout(layout &x,double sep,int apex=0);
     void make_bipartite_layout(layout &x,const ivector &p1,const ivector &p2);
     void layout_best_rotation(layout &x);
-    static rectangle layout_bounding_rect(layout &ly,double padding=0);
-    static void pack_rectangles(std::vector<rectangle> &rectangles);
-    static gen point2gen(const point &p,bool vect=false);
-    static bool gen2point(const gen &g,point &p);
-    static point layout_center(const layout &x);
-    static void scale_layout(layout &x,double diam);
-    bool is_tree() { return !is_directed() && edge_count()+1==node_count() && is_connected(); }
-    bool is_forest();
-    bool is_tournament() const;
-    bool is_planar();
+    bool is_tree(int sg=-1) { return !is_directed() && edge_count(sg)+1==subgraph_size(sg) && is_connected(sg); }
+    bool is_forest(int sg=-1,gt_conn_check cc=_GT_CC_FIND_COMPONENTS);
+    bool is_tournament(int sg=-1) const;
+    bool is_planar(int sg=-1);
     bool is_clique(int sg=-1) const;
     gen triangle_count(ivectors *dest=NULL,bool ccoeff=false,bool exact=true);
-    int tree_height(int root);
+    int tree_height(int root=-1);
     void clique_stats(std::map<int,int> &m,int mode=0);
     int maximum_clique(ivector &clique);
     void greedy_neighborhood_clique_cover_numbers(ivector &cover_numbers);
-    bool clique_cover(ivectors &cover,int k=0);
+    bool clique_cover(ivectors &cover,int k=0,int tm_lim=0,bool verbose=false);
     int maximum_independent_set(ivector &v) const;
     int girth(bool odd=false,int sg=-1);
     bool hakimi(const ivector &L);
@@ -1087,19 +1139,31 @@ public:
     void make_lcf_graph(const int *j);
     void make_sierpinski_graph(int n,int k,bool triangle);
     void make_shrikhande_graph();
-    void make_tutte_graph();
     void make_complete_graph();
     void make_complete_multipartite_graph(const ivector &partition_sizes,layout *x=NULL);
     void make_petersen_graph(int n,int k,layout *x=NULL);
-    bool make_kneser_graph(int n,int k);
+    bool make_intersection_graph(int n,int k,int sz,bool lab_sets=false);
     void make_path_graph();
     void make_cycle_graph();
-    void make_grid_graph(int m,int n,bool torus=false);
+    void make_grid_graph(int m,int n,int mode=0);
     void make_web_graph(int n,int m,layout *x=NULL);
     void make_wheel_graph(int n,layout *x=NULL);
     void make_antiprism_graph(int n,layout *x=NULL);
     void make_complete_kary_tree(int k,int d);
     bool make_flower_snark(int n,layout *x=NULL);
+    bool make_goldberg_snark(int n);
+    bool make_haar_graph(const gen &n);
+    bool make_haar_graph(ulong n);
+    void make_paley_graph(int p,int k);
+    void make_hypercube_graph(int n);
+    void make_hoffman_singleton_graph();
+    void make_sylvester_graph();
+    void make_higman_sims_graph();
+    void make_brouwer_haemers_graph();
+    void make_gewirtz_graph();
+    void make_schlaefli_graph();
+    void make_gosset_graph();
+    void make_szerekes_snark();
     void make_random_tree(int maxd=0);
     void make_random_rooted_tree();
     void make_random_free_tree();
@@ -1112,6 +1176,7 @@ public:
     void tensor_product(const graphe &G,graphe &P) const;
     void connected_components(ivectors &components,int sg=-1,bool skip_embedded=false,int *count=NULL);
     int connected_component_count(int sg=-1);
+    int connected_components_to_subgraphs(int sg=-1);
     void biconnected_components(ivectors &components,int sg=-1);
     void strongly_connected_components(ivectors &components,int sg=-1);
     bool has_cut_vertex(int sg=-1,int i=0);
@@ -1140,7 +1205,7 @@ public:
     void draw_nodes(vecteur &drawing,const layout &x) const;
     void draw_labels(vecteur &drawing,const layout &x) const;
     void distance(int i,const ivector &J,ivector &dist,ivectors *shortest_paths=NULL);
-    void allpairs_distance(matrice &m) const;
+    void allpairs_distance(matrice &m);
     void dijkstra(int src,const ivector &dest,vecteur &path_weights,ivectors *cheapest_paths=NULL,int sg=-1);
     bool bellman_ford(int src,const ivector &dest,vecteur &path_weights,ivectors *cheapest_paths=NULL);
     bool topologic_sort(ivector &ordering);
@@ -1156,10 +1221,12 @@ public:
     void parametrized_st_orientation(int s,int t,double p);
     void greedy_vertex_coloring_biggs(ivector &ordering);
     int greedy_vertex_coloring(const ivector &p);
-    int exact_vertex_coloring(int max_colors=0);
-    int exact_edge_coloring(ivector &colors,int *numcol=NULL);
-    void get_node_colors(ivector &colors);
-    bool is_bipartite(ivector &V1,ivector &V2,int sg=-1);
+    int exact_vertex_coloring(int max_colors=0,int tm_lim=0,bool verbose=false);
+    int exact_edge_coloring(ivector &colors,int *numcol=NULL,int tm_lim=0,bool verbose=false);
+    bool edge_coloring_heuristic(ivector &colors);
+    int get_node_color(int i) const;
+    void get_node_colors(ivector &colors) const;
+    bool is_bipartite(ivector &V1,ivector &V2,int sg=-1,gt_conn_check cc=_GT_CC_FIND_COMPONENTS);
     bool is_vertex_colorable(int k);
     void dsatur();
     int color_count() const;
@@ -1168,23 +1235,25 @@ public:
     ipair chromatic_number_bounds();
     void store_layout(const layout &x);
     bool has_stored_layout(layout &x) const;
-    int bipartite_matching(const ivector &p1,const ivector &p2,ipairs &matching);
+    void bipartite_matching(const ivector &p1,const ivector &p2,ipairs &matching,int sg=-1,gt_conn_check cc=_GT_CC_FIND_COMPONENTS);
+    bool weighted_bipartite_matching(const ivector &p1,const ivector &p2,ipairs &matching,
+        bool minimize=true,double eps=1e-5,int sg=-1,gt_conn_check cc=_GT_CC_FIND_COMPONENTS);
     void line_graph(graphe &G,ipairs &E) const;
     void transitive_closure(graphe &G,bool weighted=false);
-    int is_isomorphic(const graphe &other,std::map<int,int> &isom) const;
+    bool is_isomorphic(graphe &other,std::map<int,int> &isom,bool use_colors=true);
     gen aut_generators() const;
     bool canonical_labeling(ivector &lab) const;
     bool bondy_chvatal_closure(graphe &G,ivector &d);
     int hamcond(bool make_closure=true);
     bool is_hamiltonian(ivector &hc);
     bool hamcycle(ivector &path);
-    int traveling_salesman(ivector &h,double &cost,bool approximate=false);
-    bool find_directed_tours(int k,ivectors &hcv,dvector &costs,const ipairs &incl);
+    int traveling_salesman(int k,ivectors &hcv,dvector &costs,double gap_tol=0,bool verbose=true);
+    bool find_directed_tours(int k,ivectors &hcv,dvector &costs,const ipairs &incl,double gap_tol=0,bool verbose=false);
     bool make_euclidean_distances();
     gen maxflow_edmonds_karp(int s,int t,std::vector<std::map<int,gen> > &flow,const gen &limit=plusinf());
     void minimum_cut(int s,const std::vector<std::map<int,gen> > &flow,ipairs &cut);
     gen tutte_polynomial(const gen &x,const gen &y);
-    void fundamental_cycles(ivectors &cycles,int sg=-1,bool check=true);
+    void fundamental_cycles(ivectors &cycles,int sg=-1,gt_conn_check=_GT_CC_FIND_COMPONENTS);
     void mycielskian(graphe &G) const;
     gen local_clustering_coeff(int i) const;
     gen clustering_coeff(bool approx,bool exact);
@@ -1199,7 +1268,7 @@ public:
     vecteur distances_from(int k);
     gen betweenness_centrality(int k) const;
     gen communicability_betweenness_centrality(int k) const;
-    gen closeness_centrality(int k,bool harmonic=false) const;
+    gen closeness_centrality(int k,bool harmonic=false);
     gen degree_centrality(int k) const;
     vecteur katz_centrality(const gen &att) const;
     gen information_centrality(int k,bool approx=false) const;
@@ -1207,20 +1276,39 @@ public:
     bool is_split_graph(ivector &clq,ivector &indp) const;
     void contract_subgraph(graphe &G,const ivector &sg,const gen &lb) const;
     void grasp_clique(int maxitr,ivector &clq,bool cmpl=false,int sg=-1);
-    void mvc(ivector &cover,int vc_alg,int sg=0);
+    bool mvc(ivector &cover,int vc_alg,int sg=-1,int tm_lim=0,double gap_tol=0,bool verbose=false);
     int k_vertex_cover(ivector &cover,int k);
-    int vertex_cover_number();
+    int vertex_cover_number(int sg=-1);
     bool is_reachable(int u,int v);
     void reachable(int u,ivector &r);
-    void find_simplicial_vertices(ivector &res) const;
+    void find_simplicial_vertices(ivector &res);
+    void subgraph_isomorphism(graphe &P,int max_sg,bool induced,ivectors &res) const;
+    bool pruefer_encode(ivector &code,bool check_tree=false);
+    bool pruefer_decode(const ivector &code);
+    const char* identify();
+    void identify_from_sequences(vecteur &spec,int haar_limit=25);
 
     // static methods
+    static gen customize_display(int options);
+    static gen customize_layout(int options);
+    static rectangle layout_bounding_rect(layout &ly,double padding=0,int coord_offset=0);
+    static void pack_rectangles(std::vector<rectangle> &rectangles);
+    static gen point2gen(const point &p,bool vect=false);
+    static bool gen2point(const gen &g,point &p,GIAC_CONTEXT);
+    static point layout_center(const layout &x);
+    static void scale_layout(layout &x,double diam);
+    static void scale_layout_2d(layout &x,double sx,double sy);
+    static int best_quadrant(const point &p,const layout &x);
     static gen colon_label(int i,int j);
     static gen colon_label(int i,int j,int k);
     static size_t intersect_linear(ivector_iter min1,ivector_iter max1,ivector_iter min2,ivector_iter max2);
     static bool is_graphic_sequence(const ivector &s_orig);
     static ivector_iter insert_sorted(ivector &V,int val);
     static bool erase_sorted(ivector &V,int val);
+    static ivector make_ivector(int n,...);
+    static double poly_area(const layout &x);
+    static int term_hook(void *info,const char *s);
+    static bool is_interrupted();
 };
 
 #ifndef NO_NAMESPACE_GIAC
